/* 
** NetXMS - Network Management System
** Ethernet/IP support library
** Copyright (C) 2003-2020 Victor Kirhenshtein
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU Lesser General Public License as published by
** the Free Software Foundation; either version 3 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU Lesser General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
** File: main.cpp
**
**/

#include "libethernetip.h"

/**
 * Device codes
 */
static CodeLookupElement s_vendorLookupTable[] =
{
   { 1, _T("Rockwell Automation/Allen-Bradley") },
   { 2, _T("Namco Controls Corp.") },
   { 3, _T("Honeywell Inc.") },
   { 4, _T("Parker Hannifin Corp. (Veriflo Division)") },
   { 5, _T("Rockwell Automation/Reliance Elec.") },
   { 7, _T("SMC Corporation") },
   { 8, _T("Molex Incorporated") },
   { 9, _T("Western Reserve Controls Corp.") },
   { 10, _T("Advanced Micro Controls Inc. (AMCI)") },
   { 11, _T("ASCO Pneumatic Controls") },
   { 12, _T("Banner Engineering Corp.") },
   { 13, _T("Belden Wire & Cable Company") },
   { 14, _T("Cooper Interconnect") },
   { 16, _T("Daniel Woodhead Co. (Woodhead Connectivity)") },
   { 17, _T("Dearborn Group Inc.") },
   { 19, _T("Helm Instrument Company") },
   { 20, _T("Huron Net Works") },
   { 21, _T("Lumberg Inc.") },
   { 22, _T("Online Development Inc.(Automation Value)") },
   { 23, _T("Vorne Industries Inc.") },
   { 24, _T("ODVA Special Reserve") },
   { 26, _T("Festo Corporation") },
   { 30, _T("Unico Inc.") },
   { 31, _T("Ross Controls") },
   { 34, _T("Hohner Corp.") },
   { 35, _T("Micro Mo Electronics Inc.") },
   { 36, _T("MKS Instruments Inc.") },
   { 37, _T("Yaskawa Electric America formerly Magnetek Drives") },
   { 39, _T("AVG Automation (Uticor)") },
   { 40, _T("Wago Corporation") },
   { 41, _T("Kinetics (Unit Instruments)") },
   { 42, _T("IMI Norgren Limited") },
   { 43, _T("BALLUFF Inc.") },
   { 44, _T("Yaskawa Electric America Inc.") },
   { 45, _T("Eurotherm Controls Inc") },
   { 46, _T("ABB Industrial Systems") },
   { 47, _T("Omron Corporation") },
   { 48, _T("TURCk Inc.") },
   { 49, _T("Grayhill Inc.") },
   { 50, _T("Real Time Automation (C&ID)") },
   { 52, _T("Numatics Inc.") },
   { 53, _T("Lutze Inc.") },
   { 56, _T("Softing GmbH") },
   { 57, _T("Pepperl + Fuchs") },
   { 58, _T("Spectrum Controls Inc.") },
   { 59, _T("D.I.P. Inc. MKS Inst.") },
   { 60, _T("Applied Motion Products Inc.") },
   { 61, _T("Sencon Inc.") },
   { 62, _T("High Country Tek") },
   { 63, _T("SWAC Automation Consult GmbH") },
   { 64, _T("Clippard Instrument Laboratory") },
   { 68, _T("Eaton Electrical") },
   { 71, _T("Toshiba International Corp.") },
   { 72, _T("Control Technology Incorporated") },
   { 73, _T("TCS (NZ) Ltd.") },
   { 74, _T("HitachiLtd.") },
   { 75, _T("ABB Robotics Products AB") },
   { 76, _T("NKE Corporation") },
   { 77, _T("Rockwell Software Inc.") },
   { 78, _T("Escort Memory Systems (A Datalogic Group Co.)") },
   { 79, _T("Berk-Tek") },
   { 80, _T("Industrial Devices Corporation") },
   { 81, _T("IXXAT Automation GmbH") },
   { 82, _T("Mitsubishi Electric Automation Inc.") },
   { 83, _T("OPTO-22") },
   { 86, _T("Horner Electric") },
   { 87, _T("Burkert Werke GmbH & Co. KG") },
   { 88, _T("Industrial Indexing Systems, Inc.") },
   { 89, _T("Industrial Indexing Systems Inc.") },
   { 90, _T("HMS Industrial Networks AB") },
   { 91, _T("Robicon") },
   { 92, _T("Helix Technology (Granville-Phillips)") },
   { 93, _T("Arlington Laboratory") },
   { 94, _T("Advantech Co. Ltd.") },
   { 95, _T("Square D Company") },
   { 96, _T("Digital Electronics Corp.") },
   { 97, _T("Danfoss") },
   { 100, _T("Bosch Rexroth Corporation] = Pneumatics") },
   { 101, _T("Applied Materials Inc.") },
   { 102, _T("Showa Electric Wire & Cable Co.") },
   { 103, _T("Pacific Scientific (API Controls Inc.)") },
   { 104, _T("Sharp Manufacturing Systems Corp.") },
   { 105, _T("Olflex Wire & Cable Inc.") },
   { 107, _T("Unitrode") },
   { 108, _T("Beckhoff Automation GmbH") },
   { 109, _T("National Instruments") },
   { 110, _T("Mykrolis Corporations (Millipore)") },
   { 111, _T("International Motion Controls Corp.") },
   { 113, _T("SEG Kempen GmbH") },
   { 116, _T("MTS Systems Corp.") },
   { 117, _T("Krones Inc") },
   { 119, _T("EXOR Electronic R & D") },
   { 120, _T("SIEI S.p.A.") },
   { 121, _T("KUKA Roboter GmbH") },
   { 123, _T("SEC (Samsung Electronics Co.Ltd)") },
   { 124, _T("Binary Electronics Ltd") },
   { 125, _T("Flexible Machine Controls") },
   { 127, _T("ABB Inc. (Entrelec)") },
   { 128, _T("MAC Valves Inc.") },
   { 129, _T("Auma Actuators Inc") },
   { 130, _T("Toyoda Machine WorksLtd") },
   { 133, _T("Balogh T.A.G.] = Corporation") },
   { 134, _T("TR Systemtechnik GmbH") },
   { 135, _T("UNIPULSE Corporation") },
   { 138, _T("Conxall Corporation Inc.") },
   { 141, _T("Kuramo Electric Co.Ltd.") },
   { 142, _T("Creative Micro Designs") },
   { 143, _T("GE Industrial Systems") },
   { 144, _T("Leybold Vacuum GmbH") },
   { 145, _T("Siemens Energy & Automation/Drives") },
   { 146, _T("Kodensha Ltd") },
   { 147, _T("Motion Engineering Inc.") },
   { 148, _T("Honda Engineering Co.Ltd") },
   { 149, _T("EIM Valve Controls") },
   { 150, _T("Melec Inc.") },
   { 151, _T("Sony Manufacturing Systems Corporation") },
   { 152, _T("North American Mfg.") },
   { 153, _T("WATLOW") },
   { 154, _T("Japan Radio Co.Ltd") },
   { 155, _T("NADEX Co.Ltd") },
   { 156, _T("Ametek Automation & Process Technologies") },
   { 157, _T("FACTS, Inc.") },
   { 158, _T("KVASER AB") },
   { 159, _T("IDEC IZUMI Corporation") },
   { 160, _T("Mitsubishi Heavy Industries Ltd") },
   { 161, _T("Mitsubishi Electric Corporation") },
   { 162, _T("Horiba-STEC Inc.") },
   { 163, _T("esd electronic system design gmbh") },
   { 164, _T("DAIHEN Corporation") },
   { 165, _T("Tyco Valves & Controls/Keystone") },
   { 166, _T("EBARA Corporation") },
   { 169, _T("Hokuyo Electric Co. Ltd") },
   { 170, _T("Pyramid Solutions Inc.") },
   { 171, _T("Denso Wave Incorporated") },
   { 172, _T("HLS Hard-Line Solutions Inc") },
   { 173, _T("Caterpillar Inc.") },
   { 174, _T("PDL Electronics Ltd.") },
   { 176, _T("Red Lion Controls") },
   { 177, _T("ANELVA Corporation") },
   { 178, _T("Toyo Denki Seizo KK") },
   { 179, _T("Sanyo Denki Co.Ltd") },
   { 180, _T("Advanced Energy Japan K.K. (Aera Japan)") },
   { 181, _T("Pilz GmbH & Co") },
   { 182, _T("Marsh Bellofram-Bellofram PCD Division") },
   { 184, _T("M-SYSTEM Co. Ltd") },
   { 185, _T("Nissin Electric Co.Ltd") },
   { 186, _T("Hitachi Metals Ltd.") },
   { 187, _T("Oriental Motor Company") },
   { 188, _T("A&D Co.Ltd") },
   { 189, _T("Phasetronics Inc.") },
   { 190, _T("Cummins Engine Company") },
   { 191, _T("Deltron Inc.") },
   { 192, _T("Geneer Corporation") },
   { 193, _T("Anatol Automation Inc.") },
   { 196, _T("Medar Inc.") },
   { 197, _T("Comdel Inc.") },
   { 198, _T("Advanced Energy Industries Inc") },
   { 200, _T("DAIDEN Co.Ltd") },
   { 201, _T("CKD Corporation") },
   { 202, _T("Toyo Electric Corporation") },
   { 204, _T("AuCom Electronics Ltd") },
   { 205, _T("Shinko Electric Co.Ltd") },
   { 206, _T("Vector Informatik GmbH") },
   { 208, _T("Moog Inc.") },
   { 209, _T("Contemporary Controls") },
   { 210, _T("Tokyo Sokki Kenkyujo Co.Ltd") },
   { 211, _T("Schenck-AccuRate Inc.") },
   { 212, _T("The Oilgear Company") },
   { 214, _T("ASM Japan K.K.") },
   { 215, _T("HIRATA Corp.") },
   { 216, _T("SUNX Limited") },
   { 217, _T("Meidensha Corp.") },
   { 218, _T("NIDEC SANKYO CORPORATION (Sankyo Seiki Mfg. Co.Ltd)") },
   { 219, _T("KAMRO Corp.") },
   { 220, _T("Nippon System Development Co.Ltd") },
   { 221, _T("EBARA Technologies Inc.") },
   { 224, _T("SG Co.Ltd") },
   { 225, _T("Vaasa Institute of Technology") },
   { 226, _T("MKS Instruments (ENI Technology)") },
   { 227, _T("Tateyama System Laboratory Co.Ltd.") },
   { 228, _T("QLOG Corporation") },
   { 229, _T("Matric Limited Inc.") },
   { 230, _T("NSD Corporation") },
   { 232, _T("Sumitomo Wiring SystemsLtd") },
   { 233, _T("Group 3 Technology Ltd") },
   { 234, _T("CTI Cryogenics") },
   { 235, _T("POLSYS CORP") },
   { 236, _T("Ampere Inc.") },
   { 238, _T("Simplatroll Ltd") },
   { 241, _T("Leading Edge Design") },
   { 242, _T("Humphrey Products") },
   { 243, _T("Schneider Automation Inc.") },
   { 244, _T("Westlock Controls Corp.") },
   { 245, _T("Nihon Weidmuller Co.Ltd") },
   { 246, _T("Brooks Instrument (Div. of Emerson)") },
   { 248, _T("Moeller GmbH") },
   { 249, _T("Varian Vacuum Products") },
   { 250, _T("Yokogawa Electric Corporation") },
   { 251, _T("Electrical Design Daiyu Co.Ltd") },
   { 252, _T("Omron Software Co.Ltd") },
   { 253, _T("BOC Edwards") },
   { 254, _T("Control Technology Corporation") },
   { 255, _T("Bosch Rexroth") },
   { 256, _T("Turck") },
   { 257, _T("Control Techniques PLC") },
   { 258, _T("Hardy Instruments Inc.") },
   { 259, _T("LS Industrial Systems") },
   { 260, _T("E.O.A. Systems Inc.") },
   { 262, _T("New Cosmos Electric Co.Ltd.") },
   { 263, _T("Sense Eletronica LTDA") },
   { 264, _T("Xycom Inc.") },
   { 265, _T("Baldor Electric") },
   { 267, _T("Patlite Corporation") },
   { 269, _T("Mogami Wire & Cable Corporation") },
   { 270, _T("Welding Technology Corporation (WTC)") },
   { 272, _T("Deutschmann Automation GmbH") },
   { 273, _T("ICP Panel-Tec Inc.") },
   { 274, _T("Bray Controls USA") },
   { 276, _T("Status Technologies") },
   { 277, _T("Trio Motion Technology Ltd") },
   { 278, _T("Sherrex Systems Ltd") },
   { 279, _T("Adept Technology Inc.") },
   { 280, _T("Spang Power Electronics") },
   { 282, _T("Acrosser Technology Co.Ltd") },
   { 283, _T("Hilscher GmbH") },
   { 284, _T("IMAX Corporation") },
   { 285, _T("Electronic Innovation Inc. (Falter Engineering)") },
   { 286, _T("Netlogic Inc.") },
   { 287, _T("Bosch Rexroth Corporation] = Indramat") },
   { 290, _T("Murata  Machinery Ltd.") },
   { 291, _T("MTT Company Ltd.") },
   { 292, _T("Kanematsu Semiconductor Corp.") },
   { 293, _T("Takebishi Electric Sales Co.") },
   { 294, _T("Tokyo Electron Device Ltd") },
   { 295, _T("PFU Limited") },
   { 296, _T("Hakko Automation Co.Ltd.") },
   { 297, _T("Advanet Inc.") },
   { 298, _T("Tokyo Electron Software Technologies Ltd.") },
   { 300, _T("Shinagawa Electric Wire Co.Ltd.") },
   { 301, _T("Yokogawa M&C Corporation") },
   { 302, _T("KONAN Electric Co.Ltd.") },
   { 303, _T("Binar Elektronik AB") },
   { 304, _T("Furukawa Electric Co.") },
   { 305, _T("Cooper Energy Services") },
   { 306, _T("Schleicher GmbH & Co.") },
   { 307, _T("Hirose Electric Co.Ltd") },
   { 308, _T("Western Servo Design Inc.") },
   { 309, _T("Prosoft Technology") },
   { 311, _T("Towa Shoko Co.Ltd") },
   { 312, _T("Kyopal Co.Ltd") },
   { 313, _T("Extron Co.") },
   { 314, _T("Wieland Electric GmbH") },
   { 315, _T("SEW Eurodrive GmbH") },
   { 316, _T("Aera Corporation") },
   { 317, _T("STA Reutlingen") },
   { 319, _T("Fuji Electric Co.Ltd.") },
   { 322, _T("ifm efector] = inc.") },
   { 324, _T("IDEACOD-Hohner Automation S.A.") },
   { 325, _T("CommScope Inc.") },
   { 326, _T("GE Fanuc Automation North America Inc.") },
   { 327, _T("Matsushita Electric Industrial Co.Ltd") },
   { 328, _T("Okaya Electronics Corporation") },
   { 329, _T("KASHIYAMA IndustriesLtd") },
   { 330, _T("JVC") },
   { 331, _T("Interface Corporation") },
   { 332, _T("Grape Systems Inc.") },
   { 334, _T("KEBA AG") },
   { 335, _T("Toshiba IT & Control Systems Corporation") },
   { 336, _T("Sanyo Machine WorksLtd.") },
   { 337, _T("Vansco Electronics Ltd.") },
   { 338, _T("Dart Container Corp.") },
   { 339, _T("Livingston & Co. Inc.") },
   { 340, _T("Alfa Laval LKM as") },
   { 341, _T("BF ENTRON Ltd. (British Federal)") },
   { 342, _T("Bekaert Engineering NV") },
   { 343, _T("Ferran  Scientific Inc.") },
   { 344, _T("KEBA AG") },
   { 345, _T("Endress + Hauser") },
   { 346, _T("Lincoln Electric Company") },
   { 347, _T("ABB ALSTOM Power UK Ltd. (EGT)") },
   { 348, _T("Berger Lahr GmbH") },
   { 350, _T("Federal Signal Corp.") },
   { 351, _T("Kawasaki Robotics (USA) Inc.") },
   { 352, _T("Bently Nevada Corporation") },
   { 354, _T("FRABA Posital GmbH") },
   { 355, _T("Elsag Bailey Inc.") },
   { 356, _T("Fanuc Robotics America") },
   { 358, _T("Surface Combustion Inc.") },
   { 360, _T("AILES Electronics Ind. Co.Ltd.") },
   { 361, _T("Wonderware Corporation") },
   { 362, _T("Particle Measuring Systems Inc.") },
   { 365, _T("BITS Co.Ltd") },
   { 366, _T("Japan Aviation Electronics Industry Ltd") },
   { 367, _T("Keyence Corporation") },
   { 368, _T("Kuroda Precision Industries Ltd.") },
   { 369, _T("Mitsubishi Electric Semiconductor Application") },
   { 370, _T("Nippon Seisen CableLtd.") },
   { 371, _T("Omron ASO Co.Ltd") },
   { 372, _T("Seiko Seiki Co.Ltd.") },
   { 373, _T("Sumitomo Heavy IndustriesLtd.") },
   { 374, _T("Tango Computer Service Corporation") },
   { 375, _T("Technology Service Inc.") },
   { 376, _T("Toshiba Information Systems (Japan) Corporation") },
   { 377, _T("TOSHIBA Schneider Inverter Corporation") },
   { 378, _T("Toyooki Kogyo Co.Ltd.") },
   { 379, _T("XEBEC") },
   { 380, _T("Madison Cable Corporation") },
   { 381, _T("Hitati Engineering & Services Co.Ltd") },
   { 382, _T("TEM-TECH Lab Co.Ltd") },
   { 383, _T("International Laboratory Corporation") },
   { 384, _T("Dyadic Systems Co.Ltd.") },
   { 385, _T("SETO Electronics Industry Co.Ltd") },
   { 386, _T("Tokyo Electron Kyushu Limited") },
   { 387, _T("KEI System Co.Ltd") },
   { 389, _T("Asahi Engineering Co.Ltd") },
   { 390, _T("Contrex Inc.") },
   { 391, _T("Paradigm Controls Ltd.") },
   { 393, _T("Ohm Electric Co.Ltd.") },
   { 394, _T("RKC Instrument Inc.") },
   { 395, _T("Suzuki Motor Corporation") },
   { 396, _T("Custom Servo Motors Inc.") },
   { 397, _T("PACE Control Systems") },
   { 398, _T("Selectron Systems AG") },
   { 400, _T("LINTEC Co.Ltd.") },
   { 401, _T("Hitachi Cable Ltd.") },
   { 402, _T("BUSWARE Direct") },
   { 403, _T("Eaton Electric B.V. (former Holec Holland N.V.)") },
   { 404, _T("VAT Vakuumventile AG") },
   { 405, _T("Scientific Technologies Incorporated") },
   { 406, _T("Alfa Instrumentos Eletronicos Ltda") },
   { 407, _T("TWK Elektronik GmbH") },
   { 408, _T("ABB Welding Systems AB") },
   { 409, _T("BYSTRONIC Maschinen AG") },
   { 410, _T("Kimura Electric Co.Ltd") },
   { 411, _T("Nissei Plastic Industrial Co.Ltd") },
   { 413, _T("Kistler-Morse Corporation") },
   { 414, _T("Proteous Industries Inc.") },
   { 415, _T("IDC Corporation") },
   { 416, _T("Nordson Corporation") },
   { 417, _T("Rapistan Systems") },
   { 418, _T("LP-Elektronik GmbH") },
   { 419, _T("GERBI & FASE S.p.A.(Fase Saldatura)") },
   { 420, _T("Phoenix Digital Corporation") },
   { 421, _T("Z-World Engineering") },
   { 422, _T("Honda R&D Co.Ltd.") },
   { 423, _T("Bionics Instrument Co.Ltd.") },
   { 424, _T("Teknic Inc.") },
   { 425, _T("R.Stahl Inc.") },
   { 427, _T("Ryco Graphic Manufacturing Inc.") },
   { 428, _T("Giddings & Lewis Inc.") },
   { 429, _T("Koganei Corporation") },
   { 431, _T("Nichigoh Communication Electric Wire Co.Ltd.") },
   { 433, _T("Fujikura Ltd.") },
   { 434, _T("AD Link Technology Inc.") },
   { 435, _T("StoneL Corporation") },
   { 436, _T("Computer Optical Products Inc.") },
   { 437, _T("CONOS Inc.") },
   { 438, _T("Erhardt + Leimer GmbH") },
   { 439, _T("UNIQUE Co. Ltd") },
   { 440, _T("Roboticsware Inc.") },
   { 441, _T("Nachi Fujikoshi Corporation") },
   { 442, _T("Hengstler GmbH") },
   { 443, _T("Vacon Plc") },
   { 444, _T("SUNNY GIKEN Inc.") },
   { 445, _T("Lenze Drive Systems GmbH") },
   { 446, _T("CD Systems B.V.") },
   { 447, _T("FMT/Aircraft Gate Support Systems AB") },
   { 448, _T("Axiomatic Technologies Corp") },
   { 449, _T("Embedded System Products Inc.") },
   { 451, _T("Mencom Corporation") },
   { 452, _T("Kollmorgen") },
   { 453, _T("Matsushita Welding Systems Co.Ltd.") },
   { 454, _T("Dengensha Mfg. Co. Ltd.") },
   { 455, _T("Quinn Systems Ltd.") },
   { 456, _T("Tellima Technology Ltd") },
   { 457, _T("MDT] = Software") },
   { 458, _T("Taiwan Keiso Co.Ltd") },
   { 459, _T("Pinnacle Systems") },
   { 460, _T("Ascom Hasler Mailing Sys") },
   { 461, _T("INSTRUMAR Limited") },
   { 463, _T("Navistar International Transportation Corp") },
   { 464, _T("Huettinger Elektronik GmbH + Co. KG") },
   { 465, _T("OCM Technology Inc.") },
   { 466, _T("Professional Supply Inc.") },
   { 467, _T("Control Solutions") },
   { 468, _T("Baumer IVO GmbH & Co. KG") },
   { 469, _T("Worcester Controls Corporation") },
   { 470, _T("Pyramid Technical Consultants Inc.") },
   { 471, _T("Eilersen Electric A/S") },
   { 472, _T("Apollo Fire Detectors Limited") },
   { 473, _T("Avtron Manufacturing Inc.") },
   { 475, _T("Tokyo Keiso Co.Ltd.") },
   { 476, _T("Daishowa Swiki Co.Ltd.") },
   { 477, _T("Kojima Instruments Inc.") },
   { 478, _T("Shimadzu Corporation") },
   { 479, _T("Tatsuta Electric Wire & Cable Co.Ltd.") },
   { 480, _T("MECS Corporation") },
   { 481, _T("Tahara Electric") },
   { 482, _T("Koyo Electronics") },
   { 483, _T("Clever Devices") },
   { 484, _T("GCD Hardware & Software GmbH") },
   { 486, _T("Miller Electric Mfg Co.") },
   { 487, _T("GEA Tuchenhagen GmbH") },
   { 488, _T("Riken Keiki Co.] = LTD") },
   { 489, _T("Keisokugiken Corporation") },
   { 490, _T("Fuji Machine Mfg. Co.Ltd") },
   { 492, _T("Nidec-Shimpo Corp.") },
   { 493, _T("UTEC Corporation") },
   { 494, _T("Sanyo Electric Co. Ltd.") },
   { 497, _T("Okano Electric Wire Co. Ltd") },
   { 498, _T("Shimaden Co. Ltd.") },
   { 499, _T("Teddington Controls Ltd") },
   { 501, _T("VIPA GmbH") },
   { 502, _T("Warwick Manufacturing Group") },
   { 503, _T("Danaher Controls") },
   { 506, _T("American Science & Engineering") },
   { 507, _T("Accutron Controls International Inc.") },
   { 508, _T("Norcott Technologies Ltd") },
   { 509, _T("TB Woods Inc") },
   { 510, _T("Proportion-Air Inc.") },
   { 511, _T("SICK Stegmann GmbH") },
   { 513, _T("Edwards Signaling") },
   { 514, _T("Sumitomo Metal IndustriesLtd") },
   { 515, _T("Cosmo Instruments Co.Ltd.") },
   { 516, _T("Denshosha Co.Ltd.") },
   { 517, _T("Kaijo Corp.") },
   { 518, _T("Michiproducts Co.Ltd.") },
   { 519, _T("Miura Corporation") },
   { 520, _T("TG Information Network Co.Ltd.") },
   { 521, _T("Fujikin  Inc.") },
   { 522, _T("Estic Corp.") },
   { 523, _T("GS Hydraulic Sales") },
   { 524, _T("Leuze Electronic GmbH & Co. KG") },
   { 525, _T("MTE Limited") },
   { 526, _T("Hyde Park Electronics Inc.") },
   { 527, _T("Pfeiffer Vacuum GmbH") },
   { 528, _T("Cyberlogic Technologies") },
   { 529, _T("OKUMA Corporation FA Systems Division") },
   { 531, _T("Hitachi Kokusai Electric Co.Ltd.") },
   { 532, _T("SHINKO TECHNOS Co.Ltd.") },
   { 533, _T("Itoh Electric Co.Ltd.") },
   { 534, _T("Colorado Flow Tech Inc.") },
   { 535, _T("Love Controls Division/Dwyer Inst.") },
   { 536, _T("Alstom Drives and Controls") },
   { 537, _T("The Foxboro Company") },
   { 538, _T("Tescom Corporation") },
   { 540, _T("Atlas Copco Controls UK") },
   { 542, _T("Autojet Technologies") },
   { 543, _T("Prima Electronics S.p.A.") },
   { 544, _T("PMA GmbH") },
   { 545, _T("Shimafuji Electric Co.Ltd") },
   { 546, _T("Oki Electric Industry Co.Ltd") },
   { 547, _T("Kyushu Matsushita Electric Co.Ltd") },
   { 548, _T("Nihon Electric Wire & Cable Co.Ltd") },
   { 549, _T("Tsuken Electric Ind Co.Ltd") },
   { 550, _T("Tamadic Co.") },
   { 551, _T("MAATEL SA") },
   { 552, _T("OKUMA America") },
   { 553, _T("Control Techniques PLC-NA") },
   { 554, _T("TPC Wire & Cable") },
   { 555, _T("ATI Industrial Automation") },
   { 556, _T("Microcontrol (Australia) Pty Ltd") },
   { 557, _T("Serra Soldadura] = S.A.") },
   { 558, _T("Southwest Research Institute") },
   { 559, _T("Cabinplant International") },
   { 560, _T("Sartorius Mechatronics T&H GmbH") },
   { 561, _T("Comau S.p.A. Robotics & Final Assembly Division") },
   { 562, _T("Phoenix Contact") },
   { 563, _T("Yokogawa MAT Corporation") },
   { 564, _T("asahi sangyo co.] = ltd.") },
   { 566, _T("Akita Myotoku Ltd.") },
   { 567, _T("OBARA Corp.") },
   { 568, _T("Suetron Electronic GmbH") },
   { 570, _T("Serck Controls Limited") },
   { 571, _T("Fairchild Industrial Products Company") },
   { 572, _T("ARO S.A.") },
   { 573, _T("M2C GmbH") },
   { 574, _T("Shin Caterpillar Mitsubishi Ltd.") },
   { 575, _T("Santest Co.Ltd.") },
   { 576, _T("Cosmotechs Co.Ltd.") },
   { 577, _T("Hitachi Electric Systems") },
   { 578, _T("Smartscan Ltd") },
   { 579, _T("Woodhead Software & Electronics France") },
   { 580, _T("Athena Controls Inc.") },
   { 581, _T("Syron Engineering & Manufacturing Inc.") },
   { 582, _T("Asahi Optical Co.Ltd.") },
   { 583, _T("Sansha Electric Mfg. Co.Ltd.") },
   { 584, _T("Nikki Denso Co.Ltd.") },
   { 585, _T("Star Micronics] = Co.Ltd.") },
   { 586, _T("Ecotecnia Socirtat Corp.") },
   { 587, _T("AC Technology Corp.") },
   { 588, _T("West Instruments Limited") },
   { 589, _T("NTI Limited") },
   { 590, _T("Delta Computer Systems Inc.") },
   { 591, _T("FANUC Ltd.") },
   { 592, _T("Hearn-Gu Lee") },
   { 593, _T("ABB Automation Products") },
   { 594, _T("Orion Machinery Co.Ltd.") },
   { 596, _T("Wire-Pro Inc.") },
   { 597, _T("Beijing Huakong Technology Co. Ltd.") },
   { 598, _T("Yokoyama Shokai Co.Ltd.") },
   { 599, _T("Toyogiken Co.Ltd.") },
   { 600, _T("Coester Equipamentos Eletronicos Ltda.") },
   { 601, _T("Kawasaki Heavy Industries, Ltd.") },
   { 602, _T("Electroplating Engineers of Japan Ltd.") },
   { 603, _T("ROBOX S.p.A.") },
   { 604, _T("Spraying Systems Company") },
   { 605, _T("Benshaw Inc.") },
   { 606, _T("ZPA-DP A.S.") },
   { 607, _T("Wired Rite Systems") },
   { 608, _T("Tandis Research Inc.") },
   { 609, _T("SSD Drives GmbH") },
   { 610, _T("ULVAC Japan Ltd.") },
   { 611, _T("DYNAX Corporation") },
   { 612, _T("Nor-Cal Products Inc.") },
   { 613, _T("Aros Electronics AB") },
   { 614, _T("Jun-Tech Co.Ltd.") },
   { 615, _T("HAN-MI Co. Ltd.") },
   { 616, _T("uniNtech (formerly SungGi Internet)") },
   { 617, _T("Hae Pyung Electronics Reserch Institute") },
   { 618, _T("Milwaukee Electronics") },
   { 619, _T("OBERG Industries") },
   { 620, _T("Parker Hannifin/Compumotor Division") },
   { 621, _T("TECHNO DIGITAL CORPORATION") },
   { 622, _T("Network Supply Co.Ltd.") },
   { 623, _T("Union Electronics Co.Ltd.") },
   { 624, _T("Tritronics Services PM Ltd.") },
   { 625, _T("Rockwell Automation-Sprecher+Schuh") },
   { 626, _T("Matsushita Electric Industrial Co.Ltd/Motor Co.") },
   { 627, _T("Rolls-Royce Energy Systems Inc.") },
   { 628, _T("JEONGIL INTERCOM CO.] = LTD") },
   { 629, _T("Interroll Corp.") },
   { 630, _T("Hubbell Wiring Device-Kellems (Delaware)") },
   { 631, _T("Intelligent Motion Systems") },
   { 633, _T("INFICON AG") },
   { 634, _T("Hirschmann Inc.") },
   { 635, _T("The Siemon Company") },
   { 636, _T("YAMAHA Motor Co. Ltd.") },
   { 637, _T("aska corporation") },
   { 638, _T("Woodhead Connectivity") },
   { 639, _T("Trimble AB") },
   { 640, _T("Murrelektronik GmbH") },
   { 641, _T("Creatrix Labs Inc.") },
   { 642, _T("TopWorx") },
   { 643, _T("Kumho Industrial Co.Ltd.") },
   { 644, _T("Wind River Systems Inc.") },
   { 645, _T("Bihl & Wiedemann GmbH") },
   { 646, _T("Harmonic Drive Systems Inc.") },
   { 647, _T("Rikei Corporation") },
   { 648, _T("BL AutotecLtd.") },
   { 649, _T("Hana Information & Technology Co.Ltd.") },
   { 650, _T("Seoil Electric Co.Ltd.") },
   { 651, _T("Fife Corporation") },
   { 652, _T("Shanghai Electrical Apparatus Research Institute") },
   { 653, _T("Detector Electronics") },
   { 654, _T("Parasense Development Centre") },
   { 657, _T("Six Tau S.p.A.") },
   { 658, _T("Aucos GmbH") },
   { 659, _T("Rotork Controls") },
   { 660, _T("Automationdirect.com") },
   { 661, _T("Thermo BLH") },
   { 662, _T("System ControlsLtd.") },
   { 663, _T("Univer S.p.A.") },
   { 664, _T("MKS-Tenta Technology") },
   { 665, _T("Lika Electronic SNC") },
   { 666, _T("Mettler-Toledo Inc.") },
   { 667, _T("DXL USA Inc.") },
   { 668, _T("Rockwell Automation/Entek IRD Intl.") },
   { 669, _T("Nippon Otis Elevator Company") },
   { 670, _T("Sinano Electric] = Co.Ltd.") },
   { 671, _T("Sony Manufacturing Systems") },
   { 673, _T("Contec Co.Ltd.") },
   { 674, _T("Automated Solutions") },
   { 675, _T("Controlweigh") },
   { 677, _T("Fincor Electronics") },
   { 678, _T("Cognex Corporation") },
   { 679, _T("Qualiflow") },
   { 680, _T("Weidmuller Inc.") },
   { 681, _T("Morinaga Milk Industry Co.Ltd.") },
   { 682, _T("Takagi Industrial Co.Ltd.") },
   { 683, _T("Wittenstein AG") },
   { 684, _T("Sena Technologies Inc.") },
   { 686, _T("APV Products Unna") },
   { 687, _T("Creator Teknisk Utvedkling AB") },
   { 689, _T("Mibu Denki Industrial Co.Ltd.") },
   { 690, _T("Takamastsu Machineer Section") },
   { 691, _T("Startco Engineering Ltd.") },
   { 693, _T("Holjeron") },
   { 694, _T("ALCATEL High Vacuum Technology") },
   { 695, _T("Taesan LCD Co.Ltd.") },
   { 696, _T("POSCON") },
   { 697, _T("VMIC") },
   { 698, _T("Matsushita Electric WorksLtd.") },
   { 699, _T("IAI Corporation") },
   { 700, _T("Horst GmbH") },
   { 701, _T("MicroControl GmbH & Co.") },
   { 702, _T("Leine & Linde AB") },
   { 704, _T("EC Elettronica Srl") },
   { 705, _T("VIT Software HB") },
   { 706, _T("Bronkhorst High-Tech B.V.") },
   { 707, _T("Optex Co.Ltd.") },
   { 708, _T("Yosio Electronic Co.") },
   { 709, _T("Terasaki Electric Co.Ltd.") },
   { 710, _T("Sodick Co.Ltd.") },
   { 711, _T("MTS Systems Corporation-Automation Division") },
   { 712, _T("Mesa Systemtechnik") },
   { 713, _T("SHIN HO SYSTEM Co.Ltd.") },
   { 714, _T("Goyo Electronics CoLtd.") },
   { 715, _T("Loreme") },
   { 716, _T("SAB Brockskes GmbH & Co. KG") },
   { 717, _T("Trumpf Laser GmbH + Co. KG") },
   { 718, _T("Niigata Electronic Instruments Co.Ltd.") },
   { 719, _T("Yokogawa Digital Computer Corporation") },
   { 720, _T("O.N. Electronic Co.Ltd.") },
   { 721, _T("Industrial Control  Communication Inc.") },
   { 722, _T("ABB Inc.") },
   { 723, _T("ElectroWave USA Inc.") },
   { 724, _T("Industrial Network Controls] = LLC") },
   { 725, _T("KDT Systems Co.Ltd.") },
   { 726, _T("SEFA Technology Inc.") },
   { 727, _T("Nippon POP Rivets and Fasteners Ltd.") },
   { 728, _T("Yamato Scale Co.Ltd.") },
   { 729, _T("Zener Electric") },
   { 730, _T("GSE Scale Systems") },
   { 731, _T("ISAS (Integrated Switchgear & Sys. Pty Ltd)") },
   { 732, _T("Beta LaserMike Limited") },
   { 733, _T("TOEI Electric Co.Ltd.") },
   { 734, _T("Hakko Electronics Co.Ltd") },
   { 736, _T("RFID Inc.") },
   { 737, _T("Adwin Corporation") },
   { 738, _T("Osaka VacuumLtd.") },
   { 739, _T("A-Kyung Motion Inc.") },
   { 740, _T("Camozzi S.P. A.") },
   { 741, _T("Crevis Co.] = LTD") },
   { 742, _T("Rice Lake Weighing Systems") },
   { 743, _T("Linux Network Services") },
   { 744, _T("KEB Antriebstechnik GmbH") },
   { 745, _T("Hagiwara Electric Co.Ltd.") },
   { 746, _T("Glass Inc. International") },
   { 748, _T("DVT Corporation") },
   { 749, _T("Woodward Governor") },
   { 750, _T("Mosaic Systems Inc.") },
   { 751, _T("Laserline GmbH") },
   { 752, _T("COM-TEC Inc.") },
   { 753, _T("Weed Instrument") },
   { 754, _T("Prof-face European Technology Center") },
   { 755, _T("Fuji Automation Co.Ltd.") },
   { 756, _T("Matsutame Co.Ltd.") },
   { 757, _T("Hitachi Via MechanicsLtd.") },
   { 758, _T("Dainippon Screen Mfg. Co. Ltd.") },
   { 759, _T("FLS Automation A/S") },
   { 760, _T("ABB Stotz Kontakt GmbH") },
   { 761, _T("Technical Marine Service") },
   { 762, _T("Advanced Automation Associates Inc.") },
   { 763, _T("Baumer Ident GmbH") },
   { 764, _T("Tsubakimoto Chain Co.") },
   { 766, _T("Furukawa Co.Ltd.") },
   { 767, _T("Active Power") },
   { 768, _T("CSIRO Mining Automation") },
   { 769, _T("Matrix Integrated Systems") },
   { 770, _T("Digitronic Automationsanlagen GmbH") },
   { 771, _T("SICK STEGMANN Inc.") },
   { 772, _T("TAE-Antriebstechnik GmbH") },
   { 773, _T("Electronic Solutions") },
   { 774, _T("Rocon L.L.C.") },
   { 775, _T("Dijitized Communications Inc.") },
   { 776, _T("Asahi Organic Chemicals Industry Co.Ltd.") },
   { 777, _T("Hodensha") },
   { 778, _T("Harting Inc. NA") },
   { 779, _T("Kubler GmbH") },
   { 780, _T("Yamatake Corporation") },
   { 781, _T("JEOL") },
   { 782, _T("Yamatake Industrial Systems Co.Ltd.") },
   { 783, _T("HAEHNE Elektronische Messgerate GmbH") },
   { 784, _T("Ci Technologies Pty Ltd (for Pelamos Industries)") },
   { 785, _T("N. SCHLUMBERGER & CIE") },
   { 786, _T("Teijin Seiki Co.Ltd.") },
   { 787, _T("DAIKIN IndustriesLtd") },
   { 788, _T("RyuSyo Industrial Co.Ltd.") },
   { 789, _T("SAGINOMIYA SEISAKUSHO] = INC.") },
   { 790, _T("Seishin Engineering Co.Ltd.") },
   { 791, _T("Japan Support System Ltd.") },
   { 792, _T("Decsys") },
   { 793, _T("Metronix Messgerate u. Elektronik GmbH") },
   { 794, _T("ROPEX Industrie - Elektronik GmbH") },
   { 795, _T("Vaccon Company Inc.") },
   { 796, _T("Siemens Energy & Automation Inc.") },
   { 797, _T("Ten X Technology Inc.") },
   { 798, _T("Tyco Electronics") },
   { 799, _T("Delta Power Electronics Center") },
   { 800, _T("Denker") },
   { 801, _T("Autonics Corporation") },
   { 802, _T("JFE Electronic Engineering Pty. Ltd.") },
   { 804, _T("Electro-Sensors Inc.") },
   { 805, _T("Digi International Inc.") },
   { 806, _T("Texas Instruments") },
   { 807, _T("ADTEC Plasma Technology Co.Ltd") },
   { 808, _T("SICK AG") },
   { 809, _T("Ethernet Peripherals Inc.") },
   { 810, _T("Animatics Corporation") },
   { 812, _T("Process Control Corporation") },
   { 813, _T("SystemV. Inc.") },
   { 814, _T("Danaher Motion SRL") },
   { 815, _T("SHINKAWA Sensor Technology Inc.") },
   { 816, _T("Tesch GmbH & Co. KG") },
   { 818, _T("Trend Controls Systems Ltd.") },
   { 819, _T("Guangzhou ZHIYUAN Electronic Co.Ltd.") },
   { 820, _T("Mykrolis Corporation") },
   { 821, _T("Bethlehem Steel Corporation") },
   { 822, _T("KK ICP") },
   { 823, _T("Takemoto Denki Corporation") },
   { 824, _T("The Montalvo Corporation") },
   { 826, _T("LEONI Special Cables GmbH") },
   { 828, _T("ONO SOKKI CO.,LTD.") },
   { 829, _T("Rockwell Samsung Automation") },
   { 830, _T("SHINDENGEN ELECTRIC MFG. CO. LTD") },
   { 831, _T("Origin Electric Co. Ltd.") },
   { 832, _T("Quest Technical Solutions Inc.") },
   { 833, _T("LS CableLtd.") },
   { 834, _T("Enercon-Nord Electronic GmbH") },
   { 835, _T("Northwire Inc.") },
   { 836, _T("Engel Elektroantriebe GmbH") },
   { 837, _T("The Stanley Works") },
   { 838, _T("Celesco Transducer Products Inc.") },
   { 839, _T("Chugoku Electric Wire and Cable Co.") },
   { 840, _T("Kongsberg Simrad AS") },
   { 841, _T("Panduit Corporation") },
   { 842, _T("Spellman High Voltage Electronics Corp.") },
   { 843, _T("Kokusai Electric Alpha Co.Ltd.") },
   { 844, _T("Brooks Automation Inc.") },
   { 845, _T("ANYWIRE CORPORATION") },
   { 846, _T("Honda Electronics Co. Ltd") },
   { 847, _T("REO Elektronik AG") },
   { 848, _T("Fusion UV Systems Inc.") },
   { 849, _T("ASI Advanced Semiconductor Instruments GmbH") },
   { 850, _T("Datalogic Inc.") },
   { 851, _T("SoftPLC Corporation") },
   { 852, _T("Dynisco Instruments LLC") },
   { 853, _T("WEG Industrias SA") },
   { 854, _T("Frontline Test Equipment Inc.") },
   { 855, _T("Tamagawa Seiki Co.Ltd.") },
   { 856, _T("Multi Computing Co.Ltd.") },
   { 857, _T("RVSI") },
   { 858, _T("Commercial Timesharing Inc.") },
   { 859, _T("Tennessee Rand Automation LLC") },
   { 860, _T("Wacogiken Co.Ltd") },
   { 861, _T("Reflex Integration Inc.") },
   { 862, _T("Siemens AG] = A&D PI Flow Instruments") },
   { 863, _T("G. Bachmann Electronic GmbH") },
   { 864, _T("NT International") },
   { 865, _T("Schweitzer Engineering Laboratories") },
   { 866, _T("ATR Industrie-Elektronik GmbH Co.") },
   { 867, _T("PLASMATECH Co.Ltd") },
   { 869, _T("GEMU GmbH & Co. KG") },
   { 870, _T("Alcorn McBride Inc.") },
   { 871, _T("MORI SEIKI CO.] = LTD") },
   { 872, _T("NodeTech Systems Ltd") },
   { 873, _T("Emhart Teknologies") },
   { 874, _T("Cervis Inc.") },
   { 875, _T("FieldServer Technologies (Div Sierra Monitor Corp)") },
   { 876, _T("NEDAP Power Supplies") },
   { 877, _T("Nippon Sanso Corporation") },
   { 878, _T("Mitomi Giken Co.Ltd.") },
   { 879, _T("PULS GmbH") },
   { 881, _T("Japan Control Engineering Ltd") },
   { 882, _T("Embedded Systems Korea (Former Zues Emtek Co Ltd.)") },
   { 883, _T("Automa SRL") },
   { 884, _T("Harms+Wende GmbH & Co KG") },
   { 885, _T("SAE-STAHL GmbH") },
   { 886, _T("Microwave Data Systems") },
   { 887, _T("Bernecker + Rainer Industrie-Elektronik GmbH") },
   { 888, _T("Hiprom Technologies") },
   { 890, _T("Nitta Corporation") },
   { 891, _T("Kontron Modular Computers GmbH") },
   { 892, _T("Marlin Controls") },
   { 893, _T("ELCIS s.r.l.") },
   { 894, _T("Acromag Inc.") },
   { 895, _T("Avery Weigh-Tronix") },
   { 899, _T("Practicon Ltd") },
   { 900, _T("Schunk GmbH & Co. KG") },
   { 901, _T("MYNAH Technologies") },
   { 902, _T("Defontaine Groupe") },
   { 903, _T("Emerson Process Management Power & Water Solutions") },
   { 904, _T("F.A. Elec") },
   { 905, _T("Hottinger Baldwin Messtechnik GmbH") },
   { 906, _T("Coreco Imaging Inc.") },
   { 907, _T("London Electronics Ltd.") },
   { 908, _T("HSD SpA") },
   { 909, _T("Comtrol Corporation") },
   { 910, _T("TEAM] = S.A. (Tecnica Electronica de Automatismo Y Medida)") },
   { 911, _T("MAN B&W Diesel Ltd. Regulateurs Europa") },
   { 914, _T("Micro Motion Inc.") },
   { 915, _T("Eckelmann AG") },
   { 916, _T("Hanyoung Nux") },
   { 917, _T("Ransburg Industrial Finishing KK") },
   { 918, _T("Kun Hung Electric Co. Ltd.") },
   { 919, _T("Brimos wegbebakening b.v.") },
   { 920, _T("Nitto Seiki Co.Ltd") },
   { 921, _T("PPT Vision Inc.") },
   { 922, _T("Yamazaki Machinery Works") },
   { 923, _T("SCHMIDT Technology GmbH") },
   { 924, _T("Parker Hannifin SpA (SBC Division)") },
   { 925, _T("HIMA Paul Hildebrandt GmbH") },
   { 926, _T("RivaTek Inc.") },
   { 927, _T("Misumi Corporation") },
   { 928, _T("GE Multilin") },
   { 929, _T("Measurement Computing Corporation") },
   { 930, _T("Jetter AG") },
   { 931, _T("Tokyo Electronics Systems Corporation") },
   { 932, _T("Togami Electric Mfg. Co.Ltd.") },
   { 933, _T("HK Systems") },
   { 934, _T("CDA Systems Ltd.") },
   { 935, _T("Aerotech Inc.") },
   { 936, _T("JVL Industrie Elektronik A/S") },
   { 937, _T("NovaTech Process Solutions LLC") },
   { 939, _T("Cisco Systems") },
   { 940, _T("Grid Connect") },
   { 941, _T("ITW Automotive Finishing") },
   { 942, _T("HanYang System") },
   { 943, _T("ABB K.K. Technical Center") },
   { 944, _T("Taiyo Electric Wire & Cable Co.Ltd.") },
   { 946, _T("SEREN IPS INC") },
   { 947, _T("Belden CDT Electronics Division") },
   { 948, _T("ControlNet International") },
   { 949, _T("Gefran S.P.A.") },
   { 950, _T("Jokab Safety AB") },
   { 951, _T("SUMITA OPTICAL GLASS] = INC.") },
   { 952, _T("Biffi Italia srl") },
   { 953, _T("Beck IPC GmbH") },
   { 954, _T("Copley Controls Corporation") },
   { 955, _T("Fagor Automation S. Coop.") },
   { 956, _T("DARCOM") },
   { 957, _T("Frick Controls (div. of York International)") },
   { 958, _T("SymCom Inc.") },
   { 959, _T("Infranor") },
   { 960, _T("Kyosan CableLtd.") },
   { 961, _T("Varian Vacuum Technologies") },
   { 962, _T("Messung Systems") },
   { 963, _T("Xantrex Technology Inc.") },
   { 964, _T("StarThis Inc.") },
   { 965, _T("Chiyoda Co.Ltd.") },
   { 966, _T("Flowserve Corporation") },
   { 967, _T("Spyder Controls Corp.") },
   { 968, _T("IBA AG") },
   { 969, _T("SHIMOHIRA ELECTRIC MFG.CO.,LTD") },
   { 971, _T("Siemens L&A") },
   { 972, _T("Micro Innovations AG") },
   { 973, _T("Switchgear & Instrumentation") },
   { 974, _T("PRE-TECH CO.] = LTD.") },
   { 975, _T("National Semiconductor") },
   { 976, _T("Invensys Process Systems") },
   { 977, _T("Ametek HDR Power Systems") },
   { 979, _T("TETRA-K Corporation") },
   { 980, _T("C & M Corporation") },
   { 981, _T("Siempelkamp Maschinen") },
   { 983, _T("Daifuku America Corporation") },
   { 984, _T("Electro-Matic Products Inc.") },
   { 985, _T("BUSSAN MICROELECTRONICS CORP.") },
   { 986, _T("ELAU AG") },
   { 987, _T("Hetronic USA") },
   { 988, _T("NIIGATA POWER SYSTEMS Co.Ltd.") },
   { 989, _T("Software Horizons Inc.") },
   { 990, _T("B3 Systems Inc.") },
   { 991, _T("Moxa Networking Co.Ltd.") },
   { 993, _T("S4 Integration") },
   { 994, _T("Elettro Stemi S.R.L.") },
   { 995, _T("AquaSensors") },
   { 996, _T("Ifak System GmbH") },
   { 997, _T("SANKEI MANUFACTURING Co.,LTD.") },
   { 998, _T("Emerson Network Power Co.Ltd.") },
   { 999, _T("Fairmount Automation Inc.") },
   { 1000, _T("Bird Electronic Corporation") },
   { 1001, _T("Nabtesco Corporation") },
   { 1002, _T("AGM Electronics Inc.") },
   { 1003, _T("ARCX Inc.") },
   { 1004, _T("DELTA I/O Co.") },
   { 1005, _T("Chun IL Electric Ind. Co.") },
   { 1006, _T("N-Tron") },
   { 1007, _T("Nippon Pneumatics/Fludics System CO.,LTD.") },
   { 1008, _T("DDK Ltd.") },
   { 1009, _T("Seiko Epson Corporation") },
   { 1010, _T("Halstrup-Walcher GmbH") },
   { 1011, _T("ITT") },
   { 1012, _T("Ground Fault Systems bv") },
   { 1013, _T("Scolari Engineering S.p.A.") },
   { 1014, _T("Vialis Traffic bv") },
   { 1015, _T("Weidmueller Interface GmbH & Co. KG") },
   { 1016, _T("Shanghai Sibotech Automation Co. Ltd") },
   { 1017, _T("AEG Power Supply Systems GmbH") },
   { 1018, _T("Komatsu Electronics Inc.") },
   { 1019, _T("Souriau") },
   { 1020, _T("Baumuller Chicago Corp.") },
   { 1021, _T("J. Schmalz GmbH") },
   { 1022, _T("SEN Corporation") },
   { 1023, _T("Korenix Technology Co. Ltd") },
   { 1024, _T("Cooper Power Tools") },
   { 1025, _T("INNOBIS") },
   { 1026, _T("Shinho System") },
   { 1027, _T("Xm Services Ltd.") },
   { 1028, _T("KVC Co.Ltd.") },
   { 1029, _T("Sanyu Seiki Co.Ltd.") },
   { 1030, _T("TuxPLC") },
   { 1031, _T("Northern Network Solutions") },
   { 1032, _T("Converteam GmbH") },
   { 1033, _T("Symbol Technologies") },
   { 1034, _T("S-TEAM Lab") },
   { 1035, _T("Maguire Products Inc.") },
   { 1036, _T("AC&T") },
   { 1037, _T("MITSUBISHI HEAVY INDUSTRIES] = LTD. KOBE SHIPYARD & MACHINERY WORKS") },
   { 1038, _T("Hurletron Inc.") },
   { 1039, _T("Chunichi Denshi Co.Ltd") },
   { 1040, _T("Cardinal Scale Mfg. Co.") },
   { 1041, _T("BTR NETCOM via RIA Connect Inc.") },
   { 1042, _T("Base2") },
   { 1043, _T("ASRC Aerospace") },
   { 1044, _T("Beijing Stone Automation") },
   { 1045, _T("Changshu Switchgear Manufacture Ltd.") },
   { 1046, _T("METRONIX Corp.") },
   { 1047, _T("WIT") },
   { 1048, _T("ORMEC Systems Corp.") },
   { 1049, _T("ASATech (China) Inc.") },
   { 1050, _T("Controlled Systems Limited") },
   { 1051, _T("Mitsubishi Heavy Ind. Digital System Co.Ltd. (M.H.I.)") },
   { 1052, _T("Electrogrip") },
   { 1053, _T("TDS Automation") },
   { 1054, _T("T&C Power Conversion Inc.") },
   { 1055, _T("Robostar Co.Ltd") },
   { 1056, _T("Scancon A/S") },
   { 1057, _T("Haas Automation Inc.") },
   { 1058, _T("Eshed Technology") },
   { 1059, _T("Delta Electronic Inc.") },
   { 1060, _T("Innovasic Semiconductor") },
   { 1061, _T("SoftDEL Systems Limited") },
   { 1062, _T("FiberFin Inc.") },
   { 1063, _T("Nicollet Technologies Corp.") },
   { 1064, _T("B.F. Systems") },
   { 1065, _T("Empire Wire and Supply LLC") },
   { 1066, _T("ENDO KOGYO CO., LTD") },
   { 1067, _T("Elmo Motion Control LTD") },
   { 1069, _T("Asahi Keiki Co.Ltd.") },
   { 1070, _T("Joy Mining Machinery") },
   { 1071, _T("MPM Engineering Ltd") },
   { 1072, _T("Wolke Inks & Printers GmbH") },
   { 1073, _T("Mitsubishi Electric Engineering Co.Ltd.") },
   { 1074, _T("COMET AG") },
   { 1075, _T("Real Time Objects & Systems] = LLC") },
   { 1076, _T("MISCO Refractometer") },
   { 1077, _T("JT Engineering Inc.") },
   { 1078, _T("Automated Packing Systems") },
   { 1079, _T("Niobrara R&D Corp.") },
   { 1080, _T("Garmin Ltd.") },
   { 1081, _T("Japan Mobile Platform Co.Ltd") },
   { 1082, _T("Advosol Inc.") },
   { 1083, _T("ABB Global Services Limited") },
   { 1084, _T("Sciemetric Instruments Inc.") },
   { 1085, _T("Tata Elxsi Ltd.") },
   { 1086, _T("TPC Mechatronics] = Co.Ltd.") },
   { 1087, _T("Cooper Bussmann") },
   { 1088, _T("Trinite Automatisering B.V.") },
   { 1089, _T("Peek Traffic B.V.") },
   { 1090, _T("Acrison Inc") },
   { 1091, _T("Applied Robotics Inc.") },
   { 1092, _T("FireBus Systems Inc.") },
   { 1093, _T("Beijing Sevenstar Huachuang Electronics") },
   { 1094, _T("Magnetek") },
   { 1095, _T("Microscan") },
   { 1096, _T("Air Water Inc.") },
   { 1097, _T("Sensopart Industriesensorik GmbH") },
   { 1098, _T("Tiefenbach Control Systems GmbH") },
   { 1099, _T("INOXPA S.A") },
   { 1100, _T("Zurich University of Applied Sciences") },
   { 1101, _T("Ethernet Direct") },
   { 1102, _T("GSI-Micro-E Systems") },
   { 1103, _T("S-Net Automation Co.Ltd.") },
   { 1104, _T("Power Electronics S.L.") },
   { 1105, _T("Renesas Technology Corp.") },
   { 1106, _T("NSWCCD-SSES") },
   { 1107, _T("Porter Engineering Ltd.") },
   { 1108, _T("Meggitt Airdynamics Inc.") },
   { 1109, _T("Inductive Automation") },
   { 1110, _T("Neural ID") },
   { 1111, _T("EEPod LLC") },
   { 1112, _T("Hitachi Industrial Equipment Systems Co.Ltd.") },
   { 1113, _T("Salem Automation") },
   { 1114, _T("port GmbH") },
   { 1115, _T("B & PLUS") },
   { 1116, _T("Graco Inc.") },
   { 1117, _T("Altera Corporation") },
   { 1118, _T("Technology Brewing Corporation") },
   { 1121, _T("CSE Servelec") },
   { 1124, _T("Fluke Networks") },
   { 1125, _T("Tetra Pak Packaging Solutions SPA") },
   { 1126, _T("Racine Federated Inc.") },
   { 1127, _T("Pureron Japan Co.Ltd.") },
   { 1130, _T("Brother IndustriesLtd.") },
   { 1132, _T("Leroy Automation") },
   { 1134, _T("THK CO.] = LTD.") },
   { 1137, _T("TR-Electronic GmbH") },
   { 1138, _T("ASCON S.p.A.") },
   { 1139, _T("Toledo do Brasil Industria de Balancas Ltda.") },
   { 1140, _T("Bucyrus DBT Europe GmbH") },
   { 1141, _T("Emerson Process Management Valve Automation") },
   { 1142, _T("Alstom Transport") },
   { 1144, _T("Matrox Electronic Systems") },
   { 1145, _T("Littelfuse") },
   { 1146, _T("PLASMART Inc.") },
   { 1147, _T("Miyachi Corporation") },
   { 1150, _T("Promess Incorporated") },
   { 1151, _T("COPA-DATA GmbH") },
   { 1152, _T("Precision Engine Controls Corporation") },
   { 1153, _T("Alga Automacao e controle LTDA") },
   { 1154, _T("U.I. Lapp GmbH") },
   { 1155, _T("ICES") },
   { 1156, _T("Philips Lighting bv") },
   { 1157, _T("Aseptomag AG") },
   { 1158, _T("ARC Informatique") },
   { 1159, _T("Hesmor GmbH") },
   { 1160, _T("Kobe SteelLtd.") },
   { 1161, _T("FLIR Systems") },
   { 1162, _T("Simcon A/S") },
   { 1163, _T("COPALP") },
   { 1164, _T("Zypcom Inc.") },
   { 1165, _T("Swagelok") },
   { 1166, _T("Elspec") },
   { 1167, _T("ITT Water & Wastewater AB") },
   { 1168, _T("Kunbus GmbH Industrial Communication") },
   { 1170, _T("Performance Controls Inc.") },
   { 1171, _T("ACS Motion ControlLtd.") },
   { 1173, _T("IStar Technology Limited") },
   { 1174, _T("Alicat Scientific Inc.") },
   { 1176, _T("ADFweb.com SRL") },
   { 1177, _T("Tata Consultancy Services Limited") },
   { 1178, _T("CXR Ltd.") },
   { 1179, _T("Vishay Nobel AB") },
   { 1181, _T("SolaHD") },
   { 1182, _T("Endress+Hauser") },
   { 1183, _T("Bartec GmbH") },
   { 1185, _T("AccuSentry Inc.") },
   { 1186, _T("Exlar Corporation") },
   { 1187, _T("ILS Technology") },
   { 1188, _T("Control Concepts Inc.") },
   { 1190, _T("Procon Engineering Limited") },
   { 1191, _T("Hermary Opto Electronics Inc.") },
   { 1192, _T("Q-Lambda") },
   { 1194, _T("VAMP Ltd") },
   { 1195, _T("FlexLink") },
   { 1196, _T("Office FA.com Co.Ltd.") },
   { 1197, _T("SPMC (Changzhou) Co. Ltd.") },
   { 1198, _T("Anton Paar GmbH") },
   { 1199, _T("Zhuzhou CSR Times Electric Co.Ltd.") },
   { 1200, _T("DeStaCo") },
   { 1201, _T("Synrad Inc") },
   { 1202, _T("Bonfiglioli Vectron GmbH") },
   { 1203, _T("Pivotal Systems") },
   { 1204, _T("TKSCT") },
   { 1205, _T("Randy Nuernberger") },
   { 1206, _T("CENTRALP") },
   { 1207, _T("Tengen Group") },
   { 1208, _T("OES Inc.") },
   { 1209, _T("Actel Corporation") },
   { 1210, _T("Monaghan Engineering Inc.") },
   { 1211, _T("wenglor sensoric gmbh") },
   { 1212, _T("HSA Systems") },
   { 1213, _T("MK Precision Co.Ltd.") },
   { 1214, _T("Tappan Wire and Cable") },
   { 1215, _T("Heinzmann GmbH & Co. KG") },
   { 1216, _T("Process Automation International Ltd.") },
   { 1217, _T("Secure Crossing") },
   { 1218, _T("SMA Railway Technology GmbH") },
   { 1219, _T("FMS Force Measuring Systems AG") },
   { 1220, _T("ABT Endustri Enerji Sistemleri Sanayi Tic. Ltd. Sti.") },
   { 1221, _T("MagneMotion Inc.") },
   { 1222, _T("STS Co.Ltd.") },
   { 1223, _T("MERAK SIC] = SA") },
   { 1224, _T("ABOUNDI Inc.") },
   { 1225, _T("Rosemount Inc.") },
   { 1226, _T("GEA FES Inc.") },
   { 1227, _T("TMG Technologie und Engineering GmbH") },
   { 1228, _T("embeX GmbH") },
   { 1229, _T("GH Electrotermia] = S.A.") },
   { 1230, _T("Tolomatic") },
   { 1231, _T("Dukane") },
   { 1232, _T("Elco (Tian Jin) Electronics Co.Ltd.") },
   { 1233, _T("Jacobs Automation") },
   { 1234, _T("Noda Radio Frequency Technologies Co.Ltd.") },
   { 1235, _T("MSC Tuttlingen GmbH") },
   { 1236, _T("Hitachi Cable Manchester") },
   { 1237, _T("ACOREL SAS") },
   { 1238, _T("Global Engineering Solutions Co.Ltd.") },
   { 1239, _T("ALTE Transportation] = S.L.") },
   { 1240, _T("Penko Engineering B.V.") },
   { 1241, _T("Z-Tec Automation Systems Inc.") },
   { 1242, _T("ENTRON Controls LLC") },
   { 1243, _T("Johannes Huebner Fabrik Elektrischer Maschinen GmbH") },
   { 1244, _T("RF IDeas, Inc.") },
   { 1245, _T("Pentronic AB") },
   { 1246, _T("SCA Schucker GmbH & Co. KG") },
   { 1247, _T("TDK-Lambda") },
   { 1250, _T("Altronic LLC") },
   { 1251, _T("Siemens AG") },
   { 1252, _T("Liebherr Transportation Systems GmbH & Co KG") },
   { 1254, _T("SKF USA Inc.") },
   { 1256, _T("LMI Technologies") },
   { 1259, _T("EN Technologies Inc.") },
   { 1261, _T("CEPHALOS Automatisierung mbH") },
   { 1262, _T("Atronix Engineering, Inc.") },
   { 1263, _T("Monode Marking Products, Inc.") },
   { 1265, _T("Quabbin Wire & Cable Co., Inc.") },
   { 1266, _T("GPSat Systems Australia") },
   { 1269, _T("Tri-Tronics Co., Inc.") },
   { 1270, _T("Rovema GmbH") },
   { 1272, _T("IEP GmbH") },
   { 1277, _T("Control Chief Corporation") },
   { 1280, _T("Jacktek Systems Inc.") },
   { 1282, _T("PRIMES GmbH") },
   { 1283, _T("Branson Ultrasonics") },
   { 1284, _T("DEIF A/S") },
   { 1285, _T("3S-Smart Software Solutions GmbH") },
   { 1287, _T("Smarteye Corporation") },
   { 1288, _T("Toshiba Machine") },
   { 1289, _T("eWON") },
   { 1290, _T("OFS") },
   { 1291, _T("KROHNE") },
   { 1293, _T("General Cable Industries, Inc.") },
   { 1295, _T("Kistler Instrumente AG") },
   { 1296, _T("YJS Co., Ltd.") },
   { 1301, _T("Xylem Analytics Germany GmbH") },
   { 1302, _T("Lenord, Bauer & Co. GmbH") },
   { 1303, _T("Carlo Gavazzi Controls") },
   { 1304, _T("Faiveley Transport") },
   { 1306, _T("vMonitor") },
   { 1307, _T("Kepware Technologies") },
   { 1308, _T("duagon AG") },
   { 1310, _T("Xylem Water Solutions") },
   { 1311, _T("Automation Professionals, LLC") },
   { 1313, _T("CEIA SpA") },
   { 1314, _T("Marine Technologies LLC") },
   { 1315, _T("Alphagate Automatisierungstechnik GmbH") },
   { 1316, _T("Mecco Partners, LLC") },
   { 1317, _T("LAP GmbH Laser Applikationen") },
   { 1318, _T("ABB S.p.A. - SACE Division") },
   { 1319, _T("ABB S.p.A. - SACE Division") },
   { 1322, _T("Thermo Ramsey Inc.") },
   { 1323, _T("Helmholz GmbH & Co. KG") },
   { 1324, _T("EUCHNER GmbH + Co. KG") },
   { 1325, _T("AMK GmbH & Co. KG") },
   { 1326, _T("Badger Meter") },
   { 1328, _T("Fisher-Rosemount Systems, Inc.") },
   { 1329, _T("LJU Automatisierungstechnik GmbH") },
   { 1330, _T("Fairbanks Scales, Inc.") },
   { 1331, _T("Imperx, Inc.") },
   { 1332, _T("FRONIUS International GmbH") },
   { 1333, _T("Hoffman Enclosures") },
   { 1334, _T("Elecsys Corporation") },
   { 1335, _T("Bedrock Automation") },
   { 1336, _T("RACO Manufacturing and Engineering") },
   { 1337, _T("Hein Lanz Industrial Tech.") },
   { 1338, _T("Synopsys, Inc. (formerly Codenomicon)") },
   { 1341, _T("Sensirion AG") },
   { 1342, _T("SIKO GmbH") },
   { 1344, _T("GRUNDFOS") },
   { 1346, _T("Beijer Electronics Products AB") },
   { 1348, _T("AIMCO") },
   { 1350, _T("Coval Vacuum Managers") },
   { 1351, _T("Powell Industries") },
   { 1353, _T("IPDisplays") },
   { 1354, _T("SCAIME SAS") },
   { 1355, _T("Metal Work SpA") },
   { 1356, _T("Telsonic AG") },
   { 1358, _T("Hauch & Bach ApS") },
   { 1359, _T("Pago AG") },
   { 1360, _T("ULTIMATE Europe Transportation Equipment GmbH") },
   { 1362, _T("Enovation Controls") },
   { 1363, _T("Lake Cable LLC") },
   { 1367, _T("Laird") },
   { 1368, _T("Nanotec Electronic GmbH & Co. KG") },
   { 1369, _T("SAMWON ACT Co., Ltd.") },
   { 1370, _T("Aparian Inc.") },
   { 1371, _T("Cosys Inc.") },
   { 1372, _T("Insight Automation Inc.") },
   { 1374, _T("FASTECH") },
   { 1375, _T("K.A. Schmersal GmbH & Co. KG") },
   { 1377, _T("Chromalox") },
   { 1378, _T("SEIDENSHA ELECTRONICS CO., LTD") },
   { 1380, _T("Don Electronics Ltd") },
   { 1381, _T("burster gmbh & co kg") },
   { 1382, _T("Unitronics (1989) (RG) LTD") },
   { 1383, _T("OEM Technology Solutions") },
   { 1384, _T("Allied Motion") },
   { 1385, _T("Mitron Oy") },
   { 1386, _T("Dengensha TOA") },
   { 1387, _T("Systec Systemtechnik und Industrieautomation GmbH") },
   { 1389, _T("Jenny Science AG") },
   { 1390, _T("Baumer Optronic GmbH") },
   { 1391, _T("Invertek Drives Ltd") },
   { 1392, _T("High Grade Controls Corporation") },
   { 1394, _T("Ishida Europe Limited") },
   { 1396, _T("Actia Systems") },
   { 1398, _T("Beijing Tiandi-Marco Electro-Hydraulic Control System Co., Ltd.") },
   { 1399, _T("Universal Robots A/S") },
   { 1401, _T("Dialight") },
   { 1402, _T("E-T-A Elektrotechnische Apparate GmbH") },
   { 1403, _T("Kemppi Oy") },
   { 1404, _T("Tianjin Geneuo Technology Co., Ltd.") },
   { 1405, _T("ORing Industrial Networking Corp.") },
   { 1406, _T("Benchmark Electronics") },
   { 1408, _T("ELAP S.R.L.") },
   { 1409, _T("Applied Mining Technologies") },
   { 1410, _T("KITZ SCT Corporation") },
   { 1411, _T("VTEX Corporation") },
   { 1412, _T("ESYSE GmbH Embedded Systems Engineering") },
   { 1413, _T("Automation Controls") },
   { 1415, _T("Cincinnati Test Systems") },
   { 1417, _T("Zumbach Electronics Corp.") },
   { 1418, _T("Emerson Process Management") },
   { 1419, _T("CCS Inc.") },
   { 1420, _T("Videojet, Inc.") },
   { 1421, _T("Zebra Technologies") },
   { 1422, _T("Anritsu Infivis") },
   { 1423, _T("Dimetix AG") },
   { 1424, _T("General Measure (China)") },
   { 1425, _T("Fortress Interlocks") },
   { 1427, _T("Task Force Tips") },
   { 1428, _T("SERVO-ROBOT INC.") },
   { 1429, _T("Flow Devices and Systems, Inc.") },
   { 1430, _T("nLIGHT, Inc.") },
   { 1431, _T("Microchip Technology Inc.") },
   { 1432, _T("DENT Instruments") },
   { 1433, _T("CMC Industrial Electronics Ltd.") },
   { 1434, _T("Accutron Instruments Inc.") },
   { 1435, _T("Kaeser Kompressoren SE") },
   { 1436, _T("Optoelectronics") },
   { 1437, _T("Coherix, Inc.") },
   { 1438, _T("FLSmidth A/S") },
   { 1439, _T("Kyland Corporation") },
   { 1440, _T("Cole-Parmer Instrument Company") },
   { 1441, _T("Wachendorff Automation GmbH & Co., KG") },
   { 1442, _T("SMAC Moving Coil Actuators") },
   { 1444, _T("PushCorp, Inc.") },
   { 1445, _T("Fluke Process Instruments GmbH") },
   { 1446, _T("Mini Motor srl") },
   { 1447, _T("I-CON Industry Tech.") },
   { 1448, _T("Grace Engineered Products, Inc.") },
   { 1449, _T("Zaxis Inc.") },
   { 1450, _T("Lumasense Technologies") },
   { 1451, _T("Domino Printing") },
   { 1452, _T("LightMachinery Inc") },
   { 1453, _T("DEUTA-WERKE GmbH") },
   { 1454, _T("Altus Sistemas de Automação S.A.") },
   { 1455, _T("Criterion NDT") },
   { 1456, _T("InterTech Development Company") },
   { 1457, _T("Action Labs, Incorporated") },
   { 1458, _T("Perle Systems Limited") },
   { 1459, _T("Utthunga Technologies Pvt Ltd.") },
   { 1460, _T("Dong IL Vision, Co., Ltd.") },
   { 1461, _T("Wipotec Wiege-und Positioniersysteme GmbH") },
   { 1462, _T("Atos spa") },
   { 1463, _T("Solartron Metrology LTD") },
   { 1464, _T("Willowglen Systems Inc.") },
   { 1465, _T("Analog Devices") },
   { 1466, _T("Power Electronics International, Inc.") },
   { 1468, _T("Campbell Wrapper Corporatio") },
   { 1469, _T("Herkules-Resotec Elektronik GmbH") },
   { 1470, _T("aignep spa") },
   { 1471, _T("SHANGHAI CARGOA M.&E.EQUIPMENT CO.LTD") },
   { 1472, _T("PMV Automation AB") },
   { 1473, _T("K-Patents Oy") },
   { 1474, _T("Dynatronix") },
   { 1475, _T("Atop Technologies") },
   { 1476, _T("Bitronics, LLC.") },
   { 1477, _T("Delta Tau Data Systems") },
   { 1478, _T("WITZ Corporation") },
   { 1479, _T("AUTOSOL") },
   { 1480, _T("ADB Safegate") },
   { 1481, _T("VersaBuilt, Inc") },
   { 1482, _T("Visual Technologies, Inc.") },
   { 1483, _T("Artis GmbH") },
   { 1484, _T("Reliance Electric Limited") },
   { 1485, _T("Vanderlande") },
   { 1486, _T("Packet Power") },
   { 1487, _T("ima-tec gmbh") },
   { 1488, _T("Vision Automation A/S") },
   { 1489, _T("PROCENTEC BV") },
   { 1490, _T("HETRONIK GmbH") },
   { 1491, _T("Lanmark Controls Inc.") },
   { 1492, _T("profichip GmbH") },
   { 1493, _T("flexlog GmbH") },
   { 1494, _T("YUCHANGTECH") },
   { 1495, _T("Dynapower Company") },
   { 1496, _T("TAKIKAWA ENGINEERING") },
   { 1497, _T("Ingersoll Rand") },
   { 1498, _T("ASA-RT s.r.l") },
   { 1499, _T("Trumpf Laser- und Systemtectechnik Gmbh") },
   { 1500, _T("SYNTEC TECHNOLOGY CORPORATION COMPANY") },
   { 1501, _T("Rinstrum") },
   { 1502, _T("Symbotic LLC") },
   { 1503, _T("GE Healthcare Life Sciences") },
   { 1504, _T("BlueBotics SA") },
   { 1505, _T("Dynapar Corporation") },
   { 1506, _T("Blum-Novotest") },
   { 1507, _T("CIMON") },
   { 1508, _T("Dalian SeaSky Automation Co., ltd") },
   { 1509, _T("Rethink Robotics, Inc.") },
   { 1510, _T("Ingeteam") },
   { 1511, _T("TOSEI ENGINEERING CORP.") },
   { 1512, _T("SAMSON AG") },
   { 1513, _T("TGW Mechanics GmbH") },
   { 0, nullptr }
};

/**
 * Device codes
 */
static CodeLookupElement s_deviceTypeLookupTable[] =
{
   { 0, _T("Generic Device") },
   { 1, _T("Control Station") },
   { 2, _T("AC Drive Device") },
   { 3, _T("Motor Overload") },
   { 4, _T("Limit Switch") },
   { 5, _T("Inductive Proximity Switch") },
   { 6, _T("Photoelectric Sensor") },
   { 7, _T("General Purpose Discrete I/O") },
   { 8, _T("Encoder") },
   { 9, _T("Resolver") },
   { 10, _T("General Purpose Analog I/O") },
   { 11, _T("Softstart Starter") },
   { 12, _T("Communications Adapter") },
   { 13, _T("Barcode Scanner") },
   { 14, _T("Programmable Logic Controller") },
   { 16, _T("Position Controller") },
   { 17, _T("Weigh Scale") },
   { 18, _T("Message Display") },
   { 19, _T("DC Drive") },
   { 20, _T("Servo Drive") },
   { 21, _T("Contactor") },
   { 22, _T("Motor Starter") },
   { 23, _T("Softstart Starter") },
   { 24, _T("Human-Machine Interface") },
   { 25, _T("Pneumatic Valve") },
   { 26, _T("Mass Flow Controller") },
   { 27, _T("Pneumatic Valve") },
   { 28, _T("Vacuum/Pressure Gauge") },
   { 29, _T("Process Control Value") },
   { 30, _T("Residual Gas Analyzer") },
   { 31, _T("DC Power Generator") },
   { 32, _T("RF Power Generator") },
   { 33, _T("Turbomolecular Vacuum Pump") },
   { 34, _T("Encoder") },
   { 35, _T("Safety Discrete I/O Device") },
   { 36, _T("Fluid Flow Controller") },
   { 37, _T("CIP Motion Drive") },
   { 38, _T("CompoNet Repeater") },
   { 39, _T("Enhanced Mass Flow Controller") },
   { 40, _T("CIP Modbus Device") },
   { 41, _T("CIP Modbus Translator") },
   { 42, _T("Safety Analog I/O Device") },
   { 43, _T("Generic Device") },
   { 44, _T("Managed Ethernet Switch") },
   { 45, _T("CIP Motion Safety Drive") },
   { 46, _T("Safety Drive Device") },
   { 47, _T("CIP Motion Encoder") },
   { 48, _T("CIP Motion Converter") },
   { 49, _T("CIP Motion I/O") },
   { 50, _T("ControlNet Physical Layer Component") },
   { 100, _T("In-Sight Vision System") },
   { 109, _T("Specialty I/O") },
   { 115, _T("Rockwell Automation Miscellaneous") },
   { 139, _T("RFID Scanner") },
   { 123, _T("PowerFlex 755") }, // Specification says "123" but real device reports "143"
   { 127, _T("PowerFlex 40") },  // From specifications
   { 142, _T("PowerFlex 750") }, // From specifications
   { 143, _T("PowerFlex 750-Series Embedded EtherNet/IP") },
   { 146, _T("PowerMonitor 5000 Series") },
   { 150, _T("PowerFlex 525 Embedded EtherNet/IP") },
   { 151, _T("HS-DSI to EtherNet/IP") },
   { 154, _T("Software Configured Safety Relay") },
   { 155, _T("IO-Link Master Device") },
   { 171, _T("3D Sensor") },
   { 200, _T("Embedded Component") },
   { 773, _T("Cognex DataMan Reader") },
   { 0, nullptr }
};

/**
 * Device state codes
 */
static CodeLookupElement s_deviceStateLookupTable[] =
{
   { 0, _T("Non Existent") },
   { 1, _T("Self-Testing") },
   { 2, _T("Standby") },
   { 3, _T("Operational") },
   { 4, _T("Major Recoverable Fault") },
   { 5, _T("Major Unrecoverable Fault") },
   { 0, nullptr }
};

/**
 * Extended device status codes
 */
static CodeLookupElement s_extDeviceStatusLookupTable[] =
{
   { 0, _T("Self-testing or Unknown") },
   { 1, _T("Firmware Update in Progress") },
   { 2, _T("At least one faulted I/O connection") },
   { 3, _T("No I/O connections established") },
   { 4, _T("Non-Volatile Configuration bad") },
   { 5, _T("Major Fault") },
   { 6, _T("At least one I/O connection in run mode") },
   { 7, _T("At least one I/O connection established, all in idle mode") },
   { 0, nullptr }
};

/**
 * CIP general status codes
 */
static CodeLookupElement s_cipGeneralStatusLookupTable[] =
{
   { 0x00, _T("Success") },
   { 0x01, _T("Connection failure") },
   { 0x02, _T("Resource unavailable") },
   { 0x03, _T("Invalid parameter value") },
   { 0x04, _T("Path segment error") },
   { 0x05, _T("Path destination unknown") },
   { 0x06, _T("Partial transfer") },
   { 0x07, _T("Connection lost") },
   { 0x08, _T("Service not supported") },
   { 0x09, _T("Invalid attribute value") },
   { 0x0A, _T("Attribute list error") },
   { 0x0B, _T("Already in requested mode/state") },
   { 0x0C, _T("Object state conflict") },
   { 0x0D, _T("Object already exists") },
   { 0x0E, _T("Attribute not settable") },
   { 0x0F, _T("Privilege violation") },
   { 0x10, _T("Device state conflict") },
   { 0x11, _T("Reply data too large") },
   { 0x12, _T("Fragmentation of a primitive value") },
   { 0x13, _T("Not enough data") },
   { 0x14, _T("Attribute not supported") },
   { 0x15, _T("Too much data") },
   { 0x16, _T("Object does not exist") },
   { 0x17, _T("Service fragmentation sequence not in progress") },
   { 0x18, _T("No stored attribute data") },
   { 0x19, _T("Store operation failure") },
   { 0x1A, _T("Routing failure, request packet too large") },
   { 0x1B, _T("Routing failure, response packet too large") },
   { 0x1C, _T("Missing attribute list entry data") },
   { 0x1D, _T("Invalid attribute value list") },
   { 0x1E, _T("Embedded service error") },
   { 0x1F, _T("Vendor specific error") },
   { 0x20, _T("Invalid parameter") },
   { 0x21, _T("Write-once value or medium already written") },
   { 0x22, _T("Invalid Reply Received") },
   { 0x25, _T("Key failure in path") },
   { 0x26, _T("Path size invalid") },
   { 0x27, _T("Unexpected attribute in list") },
   { 0x28, _T("Invalid member ID") },
   { 0x29, _T("Member not settable") },
   { 0x2A, _T("General failure") },
   { 0, nullptr }
};

/**
 * EIP request status codes
 */
static CodeLookupElement s_protocolStatusLookupTable[] =
{
   { 0x00, _T("SUCCESS") },
   { 0x01, _T("INVALID UNSUPPORTED") },
   { 0x02, _T("INSUFFICIENT MEMORY") },
   { 0x03, _T("MALFORMED DATA") },
   { 0x64, _T("INVALID SESSION HANDLE") },
   { 0x65, _T("INVALID LENGTH") },
   { 0x69, _T("UNSUPPORTED PROTOCOL VERSION") },
   { 0, nullptr }
};

/**
 * EIP call status codes
 */
static CodeLookupElement s_callStatusLookupTable[] =
{
   { EIP_CALL_SUCCESS, _T("SUCCESS") },
   { EIP_CALL_CONNECT_FAILED, _T("CONNECT FAILED") },
   { EIP_CALL_COMM_ERROR, _T("COMMUNICATION ERROR") },
   { EIP_CALL_TIMEOUT, _T("REQUEST TIMEOUT") },
   { EIP_CALL_BAD_RESPONSE, _T("BAD RESPONSE") },
   { EIP_INVALID_EPATH, _T("INVALID EPATH") },
   { EIP_CALL_EIP_ERROR, _T("EIP ERROR") },
   { EIP_CALL_CIP_ERROR, _T("CIP ERROR") },
   { 0, nullptr }
};

/**
 * Get vendor name from CIP code
 */
const TCHAR LIBETHERNETIP_EXPORTABLE *CIP_VendorNameFromCode(int32_t code)
{
   return CodeToText(code, s_vendorLookupTable, _T("Unknown"));
}

/**
 * Get device type name from CIP code
 */
const TCHAR LIBETHERNETIP_EXPORTABLE *CIP_DeviceTypeNameFromCode(int32_t code)
{
   return CodeToText(code, s_deviceTypeLookupTable, _T("Unknown"));
}

/**
 * Get device state text from code
 */
const TCHAR LIBETHERNETIP_EXPORTABLE *CIP_DeviceStateTextFromCode(uint8_t state)
{
   return CodeToText(static_cast<int32_t>(state), s_deviceStateLookupTable, _T("Unknown"));
}

/**
 * Get CIP general status text from code
 */
const TCHAR LIBETHERNETIP_EXPORTABLE *CIP_GeneralStatusTextFromCode(CIP_GeneralStatus status)
{
   return CodeToText(static_cast<int32_t>(status), s_cipGeneralStatusLookupTable, _T("Unknown"));
}

/**
 * Get status text from code
 */
const TCHAR LIBETHERNETIP_EXPORTABLE *EIP_ProtocolStatusTextFromCode(EIP_ProtocolStatus status)
{
   return CodeToText(static_cast<int32_t>(status), s_protocolStatusLookupTable, _T("UNKNOWN"));
}

/**
 * Get call status text from code
 */
const TCHAR LIBETHERNETIP_EXPORTABLE *EIP_CallStatusTextFromCode(EIP_CallStatus status)
{
   return CodeToText(static_cast<int32_t>(status), s_callStatusLookupTable, _T("UNKNOWN"));
}

/**
 * Decode CIP device status
 */
String LIBETHERNETIP_EXPORTABLE CIP_DecodeDeviceStatus(uint16_t status)
{
   StringBuffer decode;
   if (status & CIP_DEVICE_STATUS_OWNED)
   {
      decode.append(_T("Owned"));
   }
   if (status & CIP_DEVICE_STATUS_CONFIGURED)
   {
      if (!decode.isEmpty())
         decode.append(_T(", "));
      decode.append(_T("Configured"));
   }
   if (status & CIP_DEVICE_STATUS_MINOR_RECOVERABLE_FAULT)
   {
      if (!decode.isEmpty())
         decode.append(_T(", "));
      decode.append(_T("Minor Recoverable Fault"));
   }
   if (status & CIP_DEVICE_STATUS_MINOR_UNRECOVERABLE_FAULT)
   {
      if (!decode.isEmpty())
         decode.append(_T(", "));
      decode.append(_T("Minor Unrecoverable Fault"));
   }
   if (status & CIP_DEVICE_STATUS_MAJOR_RECOVERABLE_FAULT)
   {
      if (!decode.isEmpty())
         decode.append(_T(", "));
      decode.append(_T("Major Recoverable Fault"));
   }
   if (status & CIP_DEVICE_STATUS_MAJOR_UNRECOVERABLE_FAULT)
   {
      if (!decode.isEmpty())
         decode.append(_T(", "));
      decode.append(_T("Major Unrecoverable Fault"));
   }
   return decode;
}

/**
 * Decode extended device status from status field
 */
const TCHAR LIBETHERNETIP_EXPORTABLE *CIP_DecodeExtendedDeviceStatus(uint16_t status)
{
   return CodeToText(static_cast<int32_t>((status & CIP_DEVICE_STATUS_EXTENDED_STATUS_MASK) >> 4), s_extDeviceStatusLookupTable, _T("Unknown"));
}

/**
 * Get call failure reason
 */
String EIP_Status::failureReason() const
{
   StringBuffer reason(CodeToText(static_cast<int32_t>(callStatus), s_callStatusLookupTable, _T("UNKNOWN")));
   if (callStatus == EIP_CALL_CIP_ERROR)
   {
      reason.append(_T(" - "));
      reason.append(CodeToText(static_cast<int32_t>(cipGeneralStatus), s_cipGeneralStatusLookupTable, _T("UNKNOWN")));
   }
   else if (callStatus == EIP_CALL_EIP_ERROR)
   {
      reason.append(_T(" - "));
      reason.append(CodeToText(static_cast<int32_t>(protocolStatus), s_protocolStatusLookupTable, _T("UNKNOWN")));
   }
   return reason;
}

#ifdef _WIN32

/**
 * DLL entry point
 */
BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
   if (dwReason == DLL_PROCESS_ATTACH)
      DisableThreadLibraryCalls(hInstance);
   return TRUE;
}

#endif   /* _WIN32 */
